CL_DEBUG = false; -- for internal testing only, leave it set to false!

--[[ global addon variables ]]
CRITLINE_ID =  "Critline";
CRITLINE_VERSION = "v5.0 b2";

HEADER_TEXT_COLOR  = "|cffffffff";
SUBHEADER_TEXT_COLOR  = "|cffCEA208";
BODY_TEXT_COLOR  = "|cffffffff";
HINT_TEXT_COLOR  = "|cff00ff00";

CL_MOBFILTER = {};
-----------------------------------------------------------------------------
function Critline_DataReset()
	CL_DATABASE = {};
	Critline_Summary = {["DMG"]="",["HEAL"]="",["PET"]=""};
	Critline_Initialize();
end
-----------------------------------------------------------------------------
function Critline_SettingsReset()
	CL_SETTINGS = {};
	Critline_Initialize();
end
-----------------------------------------------------------------------------
function Critline_ResetAll()
	CL_SETTINGS = {};
	CL_DATABASE = {};
	Critline_Summary = {["DMG"]="",["HEAL"]="",["PET"]=""};
	Critline_Initialize();
end
-----------------------------------------------------------------------------
function Critline_GetHighest(tree, hittype)
	--hittype is 'NORM' or 'CRIT'
	--tree is 'DMG', 'PET' or 'HEAL'
	
	local hidmg = 0;
	local spell = "n/a";
	
	tree = string.upper(tree);
	if (CL_DATABASE == nil) then 
		return hidmg, spell;
	end

	if (CL_DATABASE[tree] == nil) then 
		return hidmg, spell;
	end

	for k,v in pairs (CL_DATABASE[tree]) do
		if ( CL_DATABASE[tree][k]["Filter"] == "0" ) then
			if ( CL_DATABASE[tree][k][hittype] ~= nil ) then
				if ( CL_DATABASE[tree][k][hittype]["Amount"] > hidmg ) then
					Critline_ProcessSpell = false;
					if (CL_SETTINGS["INVERT"..tree] == "0" and (not CritlineFilters_IsSpellInFilter(k,tree))) then --not filtered and filter not inverted
						Critline_ProcessSpell = true;
					elseif (CL_SETTINGS["INVERT"..tree] == "1" and CritlineFilters_IsSpellInFilter(k,tree)) then --filtered and filter inverted
						Critline_ProcessSpell = true;
					end
					if (Critline_ProcessSpell) then
						hidmg = CL_DATABASE[tree][k][hittype]["Amount"];
						spell = k;
					end
				end
			end
		end
	end
	return hidmg, spell;
end
-----------------------------------------------------------------------------

function Critline_RebuildAllTooltips()
	Critline_Summary = {["DMG"]="",["HEAL"]="",["PET"]=""}; --have to rebuild cached data when a spell is filtered to update tooltips
	Critline_GetSummaryRichText("DMG") 
	Critline_GetSummaryRichText("HEAL")
	Critline_GetSummaryRichText("PET")
	Critline_OnUpdate();
end
-----------------------------------------------------------------------------
Critline_Summary = {["DMG"]="",["HEAL"]="",["PET"]=""};
function Critline_GetSummaryRichText(tree)
--	This is for code optimization. We only build the rich summary when a new record is hit.
--	When a new record is recorded, the engine will blank out the Critline_Summary[tree]
-- 	This causes the tree to be rebuilt when this function is called.
	--tree is 'DMG', 'PET' or 'HEAL'
	
	tree = string.upper(tree);
	if (Critline_Summary[tree] == "") then
		Critline_Summary[tree] = Critline_BuildSummaryRichText(tree);
	end
	return Critline_Summary[tree];
end
-----------------------------------------------------------------------------
function Critline_BuildSummaryRichText(tree)
	--tree is 'DMG', 'PET' or 'HEAL'
	
	tree = string.upper(tree);
	local hicrit = Critline_GetHighest(tree, "CRIT");
	local hidmg = Critline_GetHighest(tree, "NORM");
	local rtfAttack="";

	if (CL_DATABASE[tree] == nil) then 
		return SUBHEADER_TEXT_COLOR..CL_NORECORDS_TEXT..FONT_COLOR_CODE_CLOSE;
	end
	
	local crit_name = "";
	local crit_level = "";
	local crit_amount = "";
	local crit_date = "";
	
	local norm_name = "";
	local norm_level = "";
	local norm_amount = "";
	local norm_date = "";

	local formatted_entry = "";
	local summaryformat;
	
	table.sort(CL_DATABASE[tree]);

--$sn = Spell name

--$nn = Target's Name for the Normal Damage Record
--$nl = Target's Level for the Normal Damage Record
--$na = Amount of damage for the Normal spell.
--$nd = Date of Normal spell record.
--$nc = Number of times this spells has hit normal.

--$cn = Target's Name for the Crit Damage Record
--$cl = Target's Level for the Crit Damage Record
--$ca = Amount of damage for the Crit spell.
--$cd = Date of Crit spell record.
--$cc = Number of times this spells has hit Crit.
	if (CL_SETTINGS["DETAILED"] == "0") then
		--I would like these vales to be in a user editable on the settings menu (Advanced Settings)
		--BUG:14
		summaryformat = "$sn\t[$na/$ca]\n";
	else
		summaryformat = "$sn\n   Norm: $nn ($nl)\t $na\n   Crit: $cn ($cl)\t $ca\n";  --feyde...had to add brackets around damage for parsing in Fubar tooltip
	end
	
	if Critline_FuBar_Active then  --Critline_FuBar_Active is defined in each of the Critline FuBar plugins..if they are not loaded, then this will be nil
		Critline_FuBar_Tooltip = {}; --always reset on each call
	end
	
	for spell_name,v in pairs (CL_DATABASE[tree]) do
		Critline_ProcessSpell = false;
		if type(spell_name) == "number" then -- need to get rid of old 113 errors...they are stored as numbers and cause errors when compared with string names
			CL_DATABASE[tree][spell_name] = nil;
		elseif (CL_SETTINGS["INVERT"..tree] == "0" and (not CritlineFilters_IsSpellInFilter(spell_name,tree))) then --not filtered and filter not inverted
			Critline_ProcessSpell = true;
		elseif (CL_SETTINGS["INVERT"..tree] == "1" and CritlineFilters_IsSpellInFilter(spell_name,tree)) then --filtered and filter inverted
			Critline_ProcessSpell = true;
		end
		--if (not CritlineFilters_IsSpellInFilter(spell_name,tree)) then
		if (Critline_ProcessSpell) then
			--get norm_amount
			if (CL_DATABASE[tree][spell_name]["NORM"] ~= nil) then
				if ( CL_DATABASE[tree][spell_name]["NORM"]["Amount"] == hidmg ) then
					norm_amount = GREEN_FONT_COLOR_CODE..CL_DATABASE[tree][spell_name]["NORM"]["Amount"]..FONT_COLOR_CODE_CLOSE;
				else
					norm_amount = HIGHLIGHT_FONT_COLOR_CODE..CL_DATABASE[tree][spell_name]["NORM"]["Amount"]..FONT_COLOR_CODE_CLOSE;
				end
				norm_name = CL_DATABASE[tree][spell_name]["NORM"]["Target"];
				norm_level = CL_DATABASE[tree][spell_name]["NORM"]["Level"];
				norm_date = CL_DATABASE[tree][spell_name]["NORM"]["Date"];
				norm_count = CL_DATABASE[tree][spell_name]["NORM"]["Count"];
			else
				norm_name = "";
				norm_level = "";
				norm_amount = HIGHLIGHT_FONT_COLOR_CODE.."0"..FONT_COLOR_CODE_CLOSE;
				norm_date = "";
				norm_count = "";
			end

			--get crit_amount
			if (CL_DATABASE[tree][spell_name]["CRIT"] ~= nil) then
				if ( CL_DATABASE[tree][spell_name]["CRIT"]["Amount"] == hicrit ) then
					crit_amount = GREEN_FONT_COLOR_CODE..CL_DATABASE[tree][spell_name]["CRIT"]["Amount"]..FONT_COLOR_CODE_CLOSE;
				else
					crit_amount = HIGHLIGHT_FONT_COLOR_CODE..CL_DATABASE[tree][spell_name]["CRIT"]["Amount"]..FONT_COLOR_CODE_CLOSE;
				end
				crit_name = CL_DATABASE[tree][spell_name]["CRIT"]["Target"];
				crit_level = CL_DATABASE[tree][spell_name]["CRIT"]["Level"];
				crit_date = CL_DATABASE[tree][spell_name]["CRIT"]["Date"];
				crit_count = CL_DATABASE[tree][spell_name]["CRIT"]["Count"];
			else
				crit_name = "";
				crit_level = "";
				crit_amount = HIGHLIGHT_FONT_COLOR_CODE.."0"..FONT_COLOR_CODE_CLOSE;
				crit_date = "";
				crit_count = "";
			end

			formatted_entry = summaryformat;

			formatted_entry = string.gsub(formatted_entry,"$sn",spell_name);

			formatted_entry = string.gsub(formatted_entry,"$nn",norm_name);
			formatted_entry = string.gsub(formatted_entry,"$nl",norm_level);
			formatted_entry = string.gsub(formatted_entry,"$na",norm_amount);
			formatted_entry = string.gsub(formatted_entry,"$nd",norm_date);
			formatted_entry = string.gsub(formatted_entry,"$nc",norm_count);

			formatted_entry = string.gsub(formatted_entry,"$cn",crit_name);
			formatted_entry = string.gsub(formatted_entry,"$cl",crit_level);
			formatted_entry = string.gsub(formatted_entry,"$ca",crit_amount);
			formatted_entry = string.gsub(formatted_entry,"$cd",crit_date);
			formatted_entry = string.gsub(formatted_entry,"$cc",crit_count);

			rtfAttack = rtfAttack..formatted_entry;
			
			--below we will build a table for the FuBar tooltip
			--it has been formatted the same as the summaryformat above.  if summaryformat changes, then this will need to change as well
			--i am using "!!" as a left and right separator
			if Critline_FuBar_Active then --feyde..only active when ANY Critline FuBar plugin is loaded
				if CL_SETTINGS["DETAILED"] == "0" then
					tinsert(Critline_FuBar_Tooltip,spell_name.."!!["..norm_amount.."/"..crit_amount.."]");
				else
					tinsert(Critline_FuBar_Tooltip,spell_name);
					tinsert(Critline_FuBar_Tooltip,"   Norm: "..norm_name.." ("..norm_level..")!!"..norm_amount);
					tinsert(Critline_FuBar_Tooltip,"   Crit: "..crit_name.." ("..crit_level..")!!"..crit_amount);
				end
			end
		end
	end

	if ( rtfAttack == "" ) then
		if Critline_FuBar_Active then --feyde..only active when ANY Critline FuBar plugin is loaded
			tinsert(Critline_FuBar_Tooltip,SUBHEADER_TEXT_COLOR..CL_NORECORDS_TEXT..FONT_COLOR_CODE_CLOSE);
		end
		return SUBHEADER_TEXT_COLOR..CL_NORECORDS_TEXT..FONT_COLOR_CODE_CLOSE;
	end
	
	rtfAttack = strsub(rtfAttack,1,string.len(rtfAttack)-1);
	return rtfAttack;
end

function Critline_OnEvent(event)
	if event=="VARIABLES_LOADED" then
		Critline_Initialize();
		CritlineSplashFrame:EnableMouse(0)
		CritlineSplashFrame:Clear();
	end
end
-----------------------------------------------------------------------------
function Critline_OnLoad()

	local parser = ParserLib:GetInstance("1.1")
	parser:RegisterInfoType("Critline", "hit", "Critline_ParserLib_OnHit")
	parser:RegisterInfoType("Critline", "heal", "Critline_ParserLib_OnHeal")
	this:RegisterEvent("VARIABLES_LOADED")

	Critline_Message("Critline loaded.");
end
-----------------------------------------------------------------------------
function Critline_Initialize()
	Critline_Debug("Initializing...");

	--settings
	if (CL_SETTINGS == nil) then
		CL_SETTINGS = {};
	end
	if ( CL_SETTINGS["MINIMAPPOS"] == nil ) then
		CL_SETTINGS["MINIMAPPOS"] = "0";
	end
	if ( CL_SETTINGS["SHOWMINIMAP"] == nil ) then
		CL_SETTINGS["SHOWMINIMAP"] = "1";
	end
	if (CL_SETTINGS["VERSION"] == nil) then
		CL_SETTINGS["VERSION"] = CL_VERSION;
	end
	if (CL_SETTINGS["LVLADJ"] == nil) then
		CL_SETTINGS["LVLADJ"] = "0";
	end
	if (CL_SETTINGS["SCALE"] == nil) then
		CL_SETTINGS["SCALE"] = "1";
	end
	if (CL_SETTINGS["SPLASHSCALE"] == nil) then
		CL_SETTINGS["SPLASHSCALE"] = "1";
	end
	if (CL_SETTINGS["SPLASH"] == nil) then
		CL_SETTINGS["SPLASH"] = "1";
	end
	if (CL_SETTINGS["PVPONLY"] == nil) then
		CL_SETTINGS["PVPONLY"] = "0";
	end
	if (CL_SETTINGS["PLAYSOUND"] == nil) then
		CL_SETTINGS["PLAYSOUND"] = "1";
	end
	if (CL_SETTINGS["SNAPSHOT"] == nil) then
		CL_SETTINGS["SNAPSHOT"] = "0";
	end
	if ( CL_SETTINGS["DMG"] == nil ) then
		CL_SETTINGS["DMG"] = "1";
	end
	if ( CL_SETTINGS["HEAL"] == nil ) then
		CL_SETTINGS["HEAL"] = "1";
	end
	if ( CL_SETTINGS["PET"] == nil ) then
		CL_SETTINGS["PET"] = "1";
	end
	if ( CL_SETTINGS["SHOWDMG"] == nil ) then
		CL_SETTINGS["SHOWDMG"] = "1";
	end
	if ( CL_SETTINGS["SHOWHEAL"] == nil ) then
		CL_SETTINGS["SHOWHEAL"] = "1";
	end
	if ( CL_SETTINGS["SHOWPET"] == nil ) then
		CL_SETTINGS["SHOWPET"] = "1";
	end
	if ( CL_SETTINGS["INVERTDMG"] == nil ) then
		CL_SETTINGS["INVERTDMG"] = "0";
	end
	if ( CL_SETTINGS["INVERTHEAL"] == nil ) then
		CL_SETTINGS["INVERTHEAL"] = "0";
	end
	if ( CL_SETTINGS["INVERTPET"] == nil ) then
		CL_SETTINGS["INVERTPET"] = "0";
	end
	if ( CL_SETTINGS["DETAILED"] == nil ) then
		CL_SETTINGS["DETAILED"] = "0";
	end
	if ( CL_SETTINGS["FIRSTLOAD"] == nil ) then --feyde..default FIRSTLOAD to on so CritlineDisplay_OnEvent can turn on trees by class 
		CL_SETTINGS["FIRSTLOAD"] = "1";
	end
	if ( CL_SETTINGS["SPELLCOLOR"] == nil ) then
		CL_SETTINGS["SPELLCOLOR"] = {};
		CL_SETTINGS["SPELLCOLOR"]["r"] = 1;
		CL_SETTINGS["SPELLCOLOR"]["g"] = 1;
		CL_SETTINGS["SPELLCOLOR"]["b"] = 0;
	end
	if ( CL_SETTINGS["AMOUNTCOLOR"] == nil ) then
		CL_SETTINGS["AMOUNTCOLOR"] = {};
		CL_SETTINGS["AMOUNTCOLOR"]["r"] = 1;
		CL_SETTINGS["AMOUNTCOLOR"]["g"] = 1;
		CL_SETTINGS["AMOUNTCOLOR"]["b"] = 1;
	end
	
	
	
	--database
	if (CL_DATABASE == nil) then
		CL_DATABASE = {};
	end
	if (CL_DATABASE["DMG"] == nil) then
		CL_DATABASE["DMG"] = {};
	end
	if (CL_DATABASE["HEAL"] == nil) then
		CL_DATABASE["HEAL"] = {};
	end
	if (CL_DATABASE["PET"] == nil) then
		CL_DATABASE["PET"] = {};
	end
	
	--filters
	if (CL_FILTERS == nil) then
		CL_FILTERS = {};
		Critline_Debug("filters created");
	end
	if (CL_FILTERS["mobs"] == nil) then
		CL_FILTERS["mobs"] = {};
		Critline_Debug("mobs created");
	end
	if (CL_FILTERS["DMG"] == nil) then
		CL_FILTERS["DMG"] = {};
		CL_FILTERS["HEAL"] = {};
		CL_FILTERS["PET"] = {};
		Critline_Debug("spells created");
	end
	
	Critline_OnUpdate();
	Critline_Debug("Initialization Complete.");
end
-----------------------------------------------------------------------------
--I thought about just passing the 'info' structure into the Critline_HitHandle() method, but that would
--really tie us into using the ParserLib. If we ever decided to use a different Combat Parser (e.g. parser-3.0)
--we would have serious code to change. So I decided to make these next two/three functions parse the 'info' structure
--and pass what we want into the Critline_HitHandle method. This removes our heavy dependancy on the ParserLib (creates a wrapper).
function Critline_ParserLib_OnHit(event, info)
	--TODO: Need to find out what info.source returns when its our pet.
	--if (tonumber(info.source) == 103 || info.source == "pet") then
	local myPet = UnitName("pet"); --feyde
	if myPet == nil or myPet == "" then
		myPet = "-----"; --feyde..make sure myPet is not nil, otherwise string join below will error
	end
	local source = "";
	if (tonumber(info.source) == 103 ) then  --source 103 is you.
		source = "YOU";
	--elseif (info.source == myPet or info.source == myPet.."'s") then  --feyde..is our pet the source?  "'s" is English possesive...not sure how other language handle it
	elseif (string.find(info.source, myPet)) then --keith, since we cant guess all languages purul, if the source contains our pets name then...
		source = "PET";
	end
	
	if (source == "") then
		Critline_Debug(info.source.." - "..info.amount);
		return;
	end
	if (tonumber(info.skill) == 113) then
		skill = CL_DAMAGESHIELD_TEXT;
	elseif (tonumber(info.skill) == 112) then
		skill = CL_NORMAL_TEXT;
	else
		skill = info.skill;
	end
	if (tonumber(info.victim) == 103) then
		victim = CL_SELF_TEXT;--you
	else
		victim = info.victim;
	end
	
	if (CritlineFilters_IsMobInFilter(victim)) then --feyde..check for mob in filter
		return;
	end
	
	if (info.isCrit == true) then
		Critline_Debug("CRIT!");
	end
	Critline_HitHandle(source, victim, skill, 0, info.isCrit, info.amount);
	
end
-----------------------------------------------------------------------------
function Critline_ParserLib_OnHeal(event, info)
	--do pets heal?
	--feyde....there are no pet heal events, so hopefully, pets do not heal
	if (tonumber(info.source) == 103) then  --source 103 is you! (or pet?)
		if (tonumber(info.victim) == 103) then
			victim = CL_SELF_TEXT;--you
		else
			victim = info.victim;
		end
		Critline_HitHandle(CL_SELF_TEXT, victim, info.skill, 1, info.isCrit, info.amount);
	end
end
-----------------------------------------------------------------------------
function Critline_HitHandle(attacker, target, spell, isheal, iscrit, amount)
	
	if (amount == nil) then
		Critline_Debug("No Damage! exiting...");
		return;
	end

	if (target == nil) then 
		target = "??"; 
		if (UnitExists("target")) then
			target = UnitName("target");
			targetlvl = UnitLevel("target");
		else
			if (isheal == 1) then
				target = UnitName("player");
				targetlvl = UnitLevel("player");
			else
				target = "??";
				targetlvl = 0;
			end
		end
	else
		if (UnitExists("target")) then
			targetlvl = UnitLevel("target");
		else
			targetlvl = 0;
		end
	end

	if ( CL_SETTINGS["FILTER_MOBS"] == "1" ) then
		if ( Critline_IsMobInFilter(target) ) then
			return;
		end
	end

	if ( (UnitIsPlayer("target") ~= 1) and (CL_SETTINGS["PVPONLY"] == "1") ) then
		Critline_Debug("Target !=player and PvPOnly enabled, exiting...");
		return;
	end

	local leveldiff = 0;
	if (UnitLevel("player") < targetlvl) then
		leveldiff = (targetlvl - UnitLevel("player"));
	else
		leveldiff = (UnitLevel("player") - targetlvl);
	end
	Critline_Debug("Level difference: "..leveldiff);
	if ( (tonumber(CL_SETTINGS["LVLADJ"]) ~= 0) and (tonumber(CL_SETTINGS["LVLADJ"]) < leveldiff) ) then
		Critline_Debug("Target level too low and LvlAdj enabled, exiting...");
		return;
	end

	if (CL_DATABASE == nil) then
		Critline_Debug("CL_DATABASE should not be nil at this point!");
		Critline_Initialize();
	end

	if (attacker == "PET") then
		tree = "PET";
	elseif (isheal == 1) then
		tree = "HEAL";
	else
		tree = "DMG";
	end

	--exit if not recording type dmg.
	if ( CL_SETTINGS[tree] == "0" ) then
		return;
	end
	
	local hitype = "NORM";
	if (iscrit) then
		hitype = "CRIT";
	end

	if (CL_DATABASE[tree] == nil) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."]...");
		CL_DATABASE[tree] = {};
	end
	if (CL_DATABASE[tree][spell] == nil) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."]...");
		CL_DATABASE[tree][spell] = {};
	end
	if ( CL_DATABASE[tree][spell]["Filter"] == nil ) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."][Filter]...");
		CL_DATABASE[tree][spell]["Filter"] = "0";
	end
	if (CL_DATABASE[tree][spell][hitype] == nil) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."]["..hitype.."]...");
		CL_DATABASE[tree][spell][hitype] = {};
	end

	if ( CL_DATABASE[tree][spell][hitype]["Amount"] == nil ) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."]["..hitype.."][Amount]...");
		CL_DATABASE[tree][spell][hitype]["Amount"] = 0;
	end
	if ( CL_DATABASE[tree][spell][hitype]["Count"] == nil ) then
		Critline_Debug("Creating CL_DATABASE["..tree.."]["..spell.."]["..hitype.."][Count]...");
		CL_DATABASE[tree][spell][hitype]["Count"] = 0;
	end

	CL_DATABASE[tree][spell][hitype]["Count"] = CL_DATABASE[tree][spell][hitype]["Count"] + 1;


	if ( CL_DATABASE[tree][spell][hitype]["Amount"] == nil or CL_DATABASE[tree][spell][hitype]["Amount"] < amount ) then
		CL_DATABASE[tree][spell][hitype]["Amount"] = amount;
		CL_DATABASE[tree][spell][hitype]["Target"] = target;
		CL_DATABASE[tree][spell][hitype]["Level"] = targetlvl;
		CL_DATABASE[tree][spell][hitype]["Date"] = date();
		Critline_Summary[tree] = "";
		if (not CritlineFilters_IsSpellInFilter(spell,tree)) then
			Critline_DisplayNewRecord(spell, amount, iscrit);
		end
	end
end
-----------------------------------------------------------------------------
function Critline_DisplayNewRecord(spell, amount, iscrit)

	if ( iscrit == 1 ) then
		splash_msg = CL_NEW_CRIT_RECORD_MSG;
	else
		splash_msg = CL_NEW_RECORD_MSG;
	end
	if(CL_SETTINGS["SPLASH"] == "1") then
		CritlineSplashFrame:AddMessage(amount, CL_SETTINGS["AMOUNTCOLOR"]["r"], CL_SETTINGS["AMOUNTCOLOR"]["g"],CL_SETTINGS["AMOUNTCOLOR"]["b"], 1, 3);
		CritlineSplashFrame:AddMessage(format(splash_msg, spell), CL_SETTINGS["SPELLCOLOR"]["r"], CL_SETTINGS["SPELLCOLOR"]["g"], CL_SETTINGS["SPELLCOLOR"]["b"], 1, 3);
	end
	if(CL_SETTINGS["PLAYSOUND"] == "1") then 
		PlaySound("LEVELUP", 1, 1, 0, 1, 3); 
	end
	if(CL_SETTINGS["SNAPSHOT"] == "1") then 
		--To avoid protection issues, you could substitute SetAlpha(0) and SetAlpha(100) for Hide() and Show(), respectively.
		--if UIParent:IsShown() 
		--UIParent:Hide() --SetAlpha(0)
		TakeScreenshot(); 
		--UIParent:Show() --SetAlpha(100)
	end
	Critline_OnUpdate();
end
-----------------------------------------------------------------------------
function Critline_GetAboutRichText()
	return Critline_Color(HEADER_TEXT_COLOR, CRITLINE_ID.." "..CRITLINE_VERSION).."\n"..
		Critline_Color(SUBHEADER_TEXT_COLOR, "Lead Developer:").."\n"..
		Critline_Color(BODY_TEXT_COLOR, "Greenleaves (Blackhand) formally Uggh (Eredar)").."\n"..
		Critline_Color(SUBHEADER_TEXT_COLOR, "Developers:").."\n"..
		Critline_Color(BODY_TEXT_COLOR, "feyde (bloodhoof)").."\n"..
		Critline_Color(SUBHEADER_TEXT_COLOR, "Want to contribute to Critline?:").."\n"..
		Critline_Color(BODY_TEXT_COLOR, "visit http://wow-en.curse-gaming.com/downloads/addons/action-bars/critline-5-0/").."\n";
end
-----------------------------------------------------------------------------
CLOU_Handles = {};
function Critline_OnUpdate()
	for k, v in pairs(CLOU_Handles) do
		v();
	end
end
function Critline_OnUpdateRegister(newhandler)
	table.insert(CLOU_Handles, newhandler);
end
-----------------------------------------------------------------------------
--[[ misc help functions ]]
function Critline_Color(color, msg)
	if ( msg == nil ) then
		return;
	end
	return color..msg..FONT_COLOR_CODE_CLOSE;
end
-----------------------------------------------------------------------------
function Critline_Debug(message)
	if (CL_DEBUG and DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage("CL_DEBUG: "..message);
	end
end
-----------------------------------------------------------------------------
function Critline_Message(msg)
	if ( msg == nil ) then
		msg = "------------------------------";
	end
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end
-----------------------------------------------------------------------------
